package datastructures;

import java.util.HashMap;
import java.util.Map;
import java.util.Map.Entry;
import java.util.TreeSet;

public class BeliefPermutation {
    private Map<String, Boolean> beliefs = new HashMap<String, Boolean>();
    private Value value = Value.max();

    public BeliefPermutation() {

    }

    public BeliefPermutation(Integer global, Double local, String... labels) {
        value = new Value(global, local);
        for (String label : labels) {
            if (label.charAt(0) == '-') {
                beliefs.put(label.substring(1), false);
            } else {
                beliefs.put(label, true);
            }
        }
    }

    public void scaleLocalBelief(Double scalingFactor) {
        value.scaleLocalBelief(scalingFactor);
    }

    public void addBelief(String label, Boolean booleanValue) {
        beliefs.put(label, booleanValue);
    }

    public BeliefPermutation copy() {
        BeliefPermutation result = new BeliefPermutation();
        // beliefs are OK to copy by reference, because they're immutable
        result.beliefs = new HashMap<String, Boolean>(beliefs);
        // copy values before returning for isolation/security
        result.value = value.copy();
        return result;
    }

    public Map<String, Boolean> getBeliefs() {
        // copy before returning for isolation/security
        Map<String, Boolean> result = new HashMap<String, Boolean>(beliefs);
        return result;
    }

    public void setValue(Value v) {
        value = v.copy();
    }

    public void setValue(Integer global, Double local) {
        value = new Value(global, local);
    }

    public Value valueCopy() {
        // copy before returning for isolation/security
        return value.copy();
    }

    @Override
    public String toString() {
        return String.format("%s: %s", beliefs.toString(), value.toString());
    }

    @Override
    public int hashCode() {
        // kind of hacky but it works.
        String valueString = value.toString();
        TreeSet<String> sortedKeys = new TreeSet<String>(beliefs.keySet());
        for (String label : sortedKeys) {
            String booleanString = beliefs.get(label).booleanValue() ? "true" : "false";
            valueString += "|" + label + ":" + booleanString;
        }
        return valueString.hashCode();
    }

    @Override
    public boolean equals(Object other) {
        if (!(other instanceof BeliefPermutation)) {
            return false;
        }
        if (!((BeliefPermutation) other).value.equals(value)) {
            return false;
        }
        if (((BeliefPermutation) other).beliefs.size() != beliefs.size()) {
            return false;
        }
        try {
            for (Entry<String, Boolean> belief : ((BeliefPermutation) other).beliefs.entrySet()) {
                if (!beliefs.get(belief.getKey()).equals(belief.getValue())) {
                    return false;
                }
            }
        } catch (Exception e) {
            // other.belief[x] was not present in this.beliefs
            return false;
        }
        return true;
    }
}
