package queries;

import java.util.Set;

import datastructures.BeliefPermutation;
import datastructures.BeliefPermutationIndex;
import exceptions.QueryFormattingException;
import exceptions.UnknownBeliefLabelException;

/**
 * a class for defining a set of values (i.e. query constraints) you want to return from a Cumulative Measure.
 * use it by setting a=true, b=false, etc. and then passing it to CumulativeMeasure.getValue() or
 * .getMatching().
 * 
 * @author luke
 * 
 */
public class Query {
    private final QueryToken constraintTree;

    public Query(String queryString) throws QueryFormattingException {
        constraintTree = QueryStringParser.parseCriteriaString(queryString);
    }

    public Query(QueryToken constraintTree) {
        this.constraintTree = constraintTree;
    }

    // remove any items from the set that don't match the query constraints
    public Set<BeliefPermutation> filter(Set<BeliefPermutation> permutations, BeliefPermutationIndex index)
            throws UnknownBeliefLabelException {
        Set<BeliefPermutation> result = this.constraintTree.filter(index, permutations);
        return result;
    }

    public Query inverse() {
        Query result = new Query(constraintTree.inverse());
        return result;
    }
}
