package queries;

import java.util.LinkedList;

import exceptions.QueryFormattingException;

public class TokenListNode {
    TokenListNode previous = null;
    TokenListNode next = null;
    protected QueryToken token = null;

    public TokenListNode(QueryToken token) {
        this.token = token;
    }

    public TokenListNode getNext() {
        return next;
    }

    public TokenListNode getPrevious() {
        return previous;
    }

    public QueryToken getToken() {
        return token;
    }

    @Override
    public String toString() {
        return "<" + token.toString() + ">";
    }

    /**
     * split the node into many nodes, much like String.split(), except that the split terms become their own
     * TokenListNodes as well.
     * 
     * @param term
     * @return
     */
    public TokenListNode split(String divider) {
        LinkedList<QueryToken> splitTokens = token.split(divider);
        if (splitTokens == null) {
            return null;
        }

        // build the result list
        TokenListNode result = null;
        TokenListNode resultEnd = null;
        for (QueryToken token : splitTokens) {
            TokenListNode newNode = new TokenListNode(token);
            if (result == null) {
                // first node in the result list
                result = newNode;
                resultEnd = newNode;
            }
            else {
                resultEnd.next = newNode;
                newNode.previous = resultEnd;
                resultEnd = newNode;
            }
        }

        return result;
    }

    public QueryToken asTree() throws QueryFormattingException {
        return token;
    }

    /**
     * wrapper for @see QueryToken.isConjunct()
     * 
     * @return
     */
    public boolean isConjunct() {
        return token.isConjunct();
    }
}
