package test;

import static org.junit.Assert.*;

import org.junit.Test;

import exceptions.IllegalRevisionException;
import exceptions.QueryFormattingException;
import exceptions.UnknownBeliefLabelException;

/**
 * Test scenarios inspired by: N. Friedman and J.Y. Halpern. Plausibility measures: a user’s guide. In
 * Proceedings of the Eleventh conference on Uncertainty in artificial intelligence, pages 175–184. Morgan
 * Kaufmann Publishers Inc., 1995.
 */
public class Literature_FriedmanHalpernTests extends TestBase {

    /**
     * Test the A1 constraint on page 2: if A subset B, then Pl(A) <= Pl(B)
     * 
     * @throws QueryFormattingException
     * @throws UnknownBeliefLabelException
     * @throws IllegalRevisionException
     */
    @Test
    public void testA1() throws QueryFormattingException, UnknownBeliefLabelException,
            IllegalRevisionException {
        cumulative.add("p");
        cumulative.add("q");
        cumulative.add("r");

        // do some misc. revisions, just so that distributions are not even (more interesting)
        cumulative.revise("p >= (0, 0.6)");
        cumulative.revise("q <= (0, 0.1)");
        cumulative.revise("q AND (-r OR -p) <= (1, 0.1)");

        // A1: if A subset B, then Pl(A) <= Pl(B)
        //test:
        //    A = p AND q
        //    B = p
    }

    /**
     * Test the DECOMP criteria (described below)
     * 
     * @throws QueryFormattingException
     * @throws UnknownBeliefLabelException
     * @throws IllegalRevisionException
     */
    @Test
    public void testDECOMP() throws QueryFormattingException, UnknownBeliefLabelException,
            IllegalRevisionException {
        //DECOMP:
        //    if:
        //        A,B disjoint
        //        A',B' disjoint
        //        Pl(A) <= Pl(B)
        //        Pl(A') <= Pl(B')
        //    then:
        //        Pl(A union B) <= Pl(A' union B')
        //    test:
        //        A = p
        //        B = -p
        //        A' = q
        //        B' = -q
        //        Pl(A) = 0.4 (do this with revision)
        //        Pl(A') = 0.4 (do this with revision)
        cumulative.add("p");
        cumulative.add("q");

        // assert starting point
        // (0, 1/4) (0, 1/4)
        // (0, 1/4) (0, 1/4)
        assertTrue(value("p AND q", 0, 0.25));
        assertTrue(value("-p AND q", 0, 0.25));
        assertTrue(value("p AND -q", 0, 0.25));
        assertTrue(value("-p AND -q", 0, 0.25));

        cumulative.revise("p >= (0, 0.3333)");
    }
}
