package test;

import static org.junit.Assert.assertTrue;

import org.junit.Test;

import exceptions.IllegalRevisionException;
import exceptions.QueryFormattingException;
import exceptions.UnknownBeliefLabelException;

/**
 * Test scenarios inspired by: Y. Jin and M. Thielscher. Iterated belief revision, revised. Artificial
 * Intelligence, 171(1):1–18, 2007.
 */
public class Literature_JinTests extends TestBase {

    /**
     * Example 3, page 14
     * 
     * @throws QueryFormattingException
     * @throws UnknownBeliefLabelException
     * @throws IllegalRevisionException
     */
    @Test
    public void example3Test() throws QueryFormattingException, UnknownBeliefLabelException,
            IllegalRevisionException {

        cumulative.add("LiarsAreBad");
        cumulative.add("BobIsGood");
        cumulative.add("BobIsALiar");

        // All her childhood, Alice was taught by her parents that a person who has told a lie
        // is not a good person.
        cumulative.revise("LiarsAreBad");

        // So Alice believed, initially, that if Bob has told a lie then he is not a good person.
        cumulative.revise("-BobIsALiar OR -BobIsGood");

        // After her first date with Bob, she began to believe that he is a good guy. 
        cumulative.revise("BobIsGood");

        // Then a reliable friend of Alice warns her that Bob is in fact a liar, 
        // and Alice chooses to believe her
        cumulative.revise("BobIsALiar");

        // Now, should Alice still believe that Bob is a good guy?
        // According to Postulate (Rec), Alice should not challenge Bob's morality 
        // and still believe he is good, and hence disbelieve what her parents taught her.
        // But in fact it is at least as reasonable to give up the belief that Bob is good.
        // This shows that Postulate (Rec) is too strict a criterion for belief revision operators.
        assertTrue(value("BobIsGood", 0, 0.857));
        assertTrue(value("-BobIsGood", 0, 0.143));
    }
}
