package test;

import org.junit.Test;

import exceptions.IllegalRevisionException;
import exceptions.QueryFormattingException;
import exceptions.UnknownBeliefLabelException;

/**
 * Test scenarios inspired by: E. Weydert. General belief measures. In Proceedings of the Tenth international
 * conference on Uncertainty in artificial intelligence, pages 575–582. Morgan Kaufmann Publishers Inc., 1994.
 * 
 * NOTE: in comments, < refers to the weak version of <<
 */
public class Literature_WeydertTests extends TestBase {

    /**
     * I.4 Distributivity : for all v, v', w, (w * (v + v') = (w * v) + (w * v'))
     * 
     * @throws QueryFormattingException
     * @throws UnknownBeliefLabelException
     * @throws IllegalRevisionException
     */
    @Test
    public void testDistributivity() throws QueryFormattingException, UnknownBeliefLabelException,
            IllegalRevisionException {
        cumulative.add("p");
        cumulative.add("q");
        cumulative.add("r");

        // do some misc. revisions, just so that distributions are not even (more interesting)
        cumulative.revise("p >= (0, 0.6)");
        cumulative.revise("q <= (0, 0.1)");
        cumulative.revise("q AND (-r OR -p) <= (1, 0.1)");

        // TODO
    }

    /**
     * I.6 Additive monotony: for all v, v', w, if v < v' then v + w < v' + w
     * 
     * @throws QueryFormattingException
     * @throws UnknownBeliefLabelException
     * @throws IllegalRevisionException
     */
    @Test
    public void testAdditiveMonotony() throws QueryFormattingException, UnknownBeliefLabelException,
            IllegalRevisionException {
        cumulative.add("p");
        cumulative.add("q");
        cumulative.add("r");

        // do some misc. revisions, just so that distributions are not even (more interesting)
        cumulative.revise("p >= (0, 0.6)");
        cumulative.revise("q <= (0, 0.1)");
        cumulative.revise("q AND (-r OR -p) <= (1, 0.1)");

        // TODO
    }

    /**
     * I.7 Multiplicative monotony: for all v, v', w, if v << v' & w != n then v * w << v' w
     * 
     * @throws QueryFormattingException
     * @throws UnknownBeliefLabelException
     * @throws IllegalRevisionException
     */
    @Test
    public void testMultiplicativeMonotony() throws QueryFormattingException, UnknownBeliefLabelException,
            IllegalRevisionException {
        cumulative.add("p");
        cumulative.add("q");
        cumulative.add("r");

        // do some misc. revisions, just so that distributions are not even (more interesting)
        cumulative.revise("p >= (0, 0.6)");
        cumulative.revise("q <= (0, 0.1)");
        cumulative.revise("q AND (-r OR -p) <= (1, 0.1)");

        // TODO
    }

    /**
     * I.8 Additive accessibility: for all v, v', if v < v' then there exists a w such that v + w = v'
     * 
     * @throws QueryFormattingException
     * @throws UnknownBeliefLabelException
     * @throws IllegalRevisionException
     */
    @Test
    public void testAdditiveAccessibility() throws QueryFormattingException, UnknownBeliefLabelException,
            IllegalRevisionException {
        cumulative.add("p");
        cumulative.add("q");
        cumulative.add("r");

        // do some misc. revisions, just so that distributions are not even (more interesting)
        cumulative.revise("p >= (0, 0.6)");
        cumulative.revise("q <= (0, 0.1)");
        cumulative.revise("q AND (-r OR -p) <= (1, 0.1)");

        // TODO
    }

    /**
     * I.9 Multiplicative accessibility: for all v, v', if v < v' then there exists a w such that v' * w = v
     * 
     * @throws QueryFormattingException
     * @throws UnknownBeliefLabelException
     * @throws IllegalRevisionException
     */
    @Test
    public void testMultiplicativeAccessibility() throws QueryFormattingException,
            UnknownBeliefLabelException, IllegalRevisionException {
        cumulative.add("p");
        cumulative.add("q");
        cumulative.add("r");

        // do some misc. revisions, just so that distributions are not even (more interesting)
        cumulative.revise("p >= (0, 0.6)");
        cumulative.revise("q <= (0, 0.1)");
        cumulative.revise("q AND (-r OR -p) <= (1, 0.1)");

        // TODO
    }
}
