package test;

import static org.junit.Assert.*;

import datastructures.Value;

import org.junit.Test;

import queries.Query;
import exceptions.QueryFormattingException;
import exceptions.UnknownBeliefLabelException;

public class QueryValueTests extends TestBase {
    @Test
    public void add() {
        // only want to make sure it doesn't throw any exceptions
        cumulative.add("a");
        cumulative.add("b");
    }

    /**
     * the simple version of set/get - tests simple add/get operations by a single constraint
     * 
     * @throws QueryFormattingException
     * @throws UnknownBeliefLabelException
     */
    @Test
    public void getValue() throws QueryFormattingException, UnknownBeliefLabelException {
        // any given label should be equally distributed between true (0.5) and
        // false (0.5).
        cumulative.add("a");
        assertQueryResultValue("a", 0, 0.5);

        // after adding a second label, relative distribution between any two
        // true/false for a given label should still be 0.5/0.5.
        cumulative.add("b");
        assertQueryResultValue("a", 0, 0.5);
        assertQueryResultValue("b", 0, 0.5);
        assertQueryResultValue("-a", 0, 0.5);
        assertQueryResultValue("-b", 0, 0.5);
    }

    /**
     * more complicated queries - tests more complicated query structures
     * 
     * @throws UnknownBeliefLabelException
     */
    @Test
    public void query() throws QueryFormattingException, UnknownBeliefLabelException {
        cumulative.add("a");
        cumulative.add("b");
        assertQueryResultValue("a AND b", 0, 0.25);
        assertQueryResultValue("a AND -b", 0, 0.25);
        assertQueryResultValue("-a OR b", 0, 0.75);
        assertQueryResultValue("-a OR -b", 0, 0.75);

        assertQueryInverseValue("a AND b", 0, 0.75);
        assertQueryInverseValue("-a OR -b", 0, 0.25);

        cumulative.add("c");
        cumulative.add("d");
        assertQueryResultValue("a", 0, 0.5);
        assertQueryResultValue("b", 0, 0.5);
        assertQueryResultValue("-a", 0, 0.5);
        assertQueryResultValue("-b", 0, 0.5);

        assertQueryResultValue("a OR -b", 0, 0.75);
        assertQueryInverseValue("a OR -b", 0, 0.25);
        assertQueryResultValue("a OR (-b AND c)", 0, 0.625);

        // more coverage is in QueryTests.java
    }

    private void assertQueryResultValue(String queryString, int global, double local)
            throws QueryFormattingException, UnknownBeliefLabelException {
        assertEquals(cumulative.getValue(new Query(queryString)).global().intValue(), global);
        assertEquals(cumulative.getValue(new Query(queryString)).local().doubleValue(), local,
                Value.EQUALITY_THRESHOLD);
    }

    private void assertQueryInverseValue(String queryString, int global, double local)
            throws QueryFormattingException, UnknownBeliefLabelException {
        assertEquals(cumulative.getValue(new Query(queryString).inverse()).global().intValue(), global);
        assertEquals(cumulative.getValue(new Query(queryString).inverse()).local().doubleValue(), local,
                Value.EQUALITY_THRESHOLD);
    }
}
