package test;

import static org.junit.Assert.fail;

import java.util.Set;

import org.junit.After;
import org.junit.Before;

import queries.Query;

import datastructures.BeliefPermutation;
import datastructures.CumulativeMeasure;
import datastructures.Value;
import exceptions.QueryAmbiguityException;
import exceptions.QueryConjunctPairingException;
import exceptions.QueryFormattingException;
import exceptions.UnknownBeliefLabelException;

public abstract class TestBase {

    protected CumulativeMeasure cumulative;

    @Before
    public void setUp() throws Exception {
        cumulative = new CumulativeMeasure();
    }

    @After
    public void tearDown() throws Exception {
    }

    // shorthand function
    protected Value v(Integer global, Double local) {
        return new Value(global, local);
    }

    protected void assertBadFormat(String valueString) {
        try {
            new Value(valueString);
            fail("Should have thrown QueryFormattingException");
        }
        catch (QueryFormattingException e) {
            // pass
        }
    }

    protected boolean value(String queryString, int global, double local)
            throws QueryFormattingException, UnknownBeliefLabelException {
        Value v = cumulative.getValue(new Query(queryString));
        return v.equals(new Value(global, local));
    }

    protected boolean ambiguous(String queryString) throws UnknownBeliefLabelException,
            QueryFormattingException {
        try {
            cumulative.query(queryString);
            return false;
        }
        catch (QueryAmbiguityException e) {
            // pass
        }
        return true;
    }

    protected boolean badConjunct(String queryString) throws QueryFormattingException,
            UnknownBeliefLabelException {
        try {
            cumulative.query(queryString);
            return false;
        }
        catch (QueryConjunctPairingException e) {
            // pass
        }
        return true;
    }

    protected boolean result(String queryString, BeliefPermutation[] resultSet)
            throws QueryFormattingException, UnknownBeliefLabelException {
        Set<BeliefPermutation> matches = null;
        matches = cumulative.query(queryString);
        if (resultSet.length != matches.size()) {
            return false;
        }
        for (BeliefPermutation result : resultSet) {
            if (!matches.contains(result)) {
                return false;
            }
        }
        return true;
    }
}
