package test;

import static org.junit.Assert.*;

import org.junit.Test;

import datastructures.Value;
import exceptions.QueryFormattingException;

public class ValueTests extends TestBase {
    @Test
    public void init() throws QueryFormattingException {
        Value v = v(0, 0.0);
        assertEquals(v.global(), new Integer(0));
        assertEquals(v.local(), new Double(0));

        assertBadFormat("0,0");
        assertBadFormat("(0.0, 0)");
        assertBadFormat("(0.2, 1.5)");

    }

    @Test
    public void greaterThanLessThan() throws QueryFormattingException {
        Value v0_05 = v(0, 0.5);
        Value v0_075 = v(0, 0.75);

        Value v1_5 = v(1, 5.0);
        Value v1_025 = v(1, 0.25);

        Value v2_05 = v(2, 0.5);
        Value v2_2 = v(2, 2.0);
        Value v2_10 = v(2, 10.0);

        // rank 0 vs 0
        assertTrue(v0_075.greaterThan(v0_05));
        assertTrue(v0_05.lessThan(v0_075));

        // rank 1 vs 1
        assertTrue(v1_5.greaterThan(v1_025));
        assertTrue(v1_025.lessThan(v1_5));

        // rank 2 vs 2
        assertTrue(v2_2.greaterThan(v2_05));
        assertTrue(v2_10.greaterThan(v2_2));

        assertTrue(v2_05.lessThan(v2_2));
        assertTrue(v2_2.lessThan(v2_10));

        // rank 0 vs 1
        assertTrue(v0_05.greaterThan(v1_5));
        assertTrue(v0_075.greaterThan(v1_025));

        assertTrue(v1_5.lessThan(v0_05));
        assertTrue(v1_025.lessThan(v0_05));

        // 1 vs 2
        assertTrue(v1_025.greaterThan(v2_05));
        assertTrue(v1_5.greaterThan(v2_10));

        assertTrue(v2_2.lessThan(v2_10));
        assertTrue(v2_05.lessThan(v1_5));

        // assert false (just a few reversals of the previous ones)
        assertFalse(v0_05.greaterThan(v0_075));
        assertFalse(v0_075.lessThan(v0_05));

        assertFalse(v1_025.greaterThan(v0_075));
        assertFalse(v0_05.lessThan(v1_5));

        assertFalse(v2_10.greaterThan(v1_5));
        assertFalse(v2_10.lessThan(v2_2));
    }

    @Test
    public void equalTo() throws QueryFormattingException {
        Value v1, v2;

        v1 = v(0, 5000.0 / 10000);
        v2 = v(0, 0.5);
        assertEquals(v1, v2);
        assertEquals(v2, v1);

        v1 = v(1, 0.5);
        v2 = v(1, 0.5);
        assertEquals(v1, v2);
        assertEquals(v2, v1);

        v1 = v(2, 0.5);
        v2 = v(2, 0.5);
        assertEquals(v1, v2);
        assertEquals(v2, v1);

        v2 = v(1, 0.5);
        assertFalse(v1.equals(v2));
        assertFalse(v2.equals(v1));

        v1 = v(0, 0.5);
        assertFalse(v1.equals(v2));
        assertFalse(v2.equals(v1));
    }

    @Test
    public void multiplyDivide() {
        assertEquals(v(1, 0.5).dividedBy(v(0, 0.5)), v(1, 1.0));
        assertEquals(v(1, 1.0).multipliedBy(v(0, 0.5)), v(1, 0.5));
    }
}
